@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_plugin.c,v 1.82 2004/12/21 22:57:51 olivier Exp $
+/*      $Id: xfwm4_plugin.c 16068 2005-06-25 21:51:21Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -772,26 +772,35 @@ update_theme_dir (const gchar * theme_dir, GList * theme_list)
 
     if (info)
     {
-        if (!has_decoration && !has_keybinding)
+        if (!strcmp (theme_dir, info->path))
         {
-            list = g_list_remove (list, info);
-            theme_info_free (info);
-        }
-        else if ((info->has_keybinding != has_keybinding)
-            || (info->has_decoration != has_decoration) || (info->set_layout != set_layout) || (info->set_align != set_align) || (info->set_font != set_font))
-        {
-            info->has_keybinding = has_keybinding;
-            info->has_decoration = has_decoration;
-            info->set_layout = set_layout;
-            info->set_align = set_align;
-            info->set_font = set_font;
-            info->user_writable = user_writable;
+            if (!has_decoration && !has_keybinding)
+            {
+                TRACE ("Removing %s", theme_name);
+                list = g_list_remove (list, info);
+                theme_info_free (info);
+            }
+            else if ((info->has_keybinding != has_keybinding)
+                     || (info->has_decoration != has_decoration) 
+                     || (info->set_layout != set_layout) 
+                     || (info->set_align != set_align) 
+                     || (info->set_font != set_font))
+            {
+                TRACE ("Updating %s", theme_name);
+                info->has_keybinding = has_keybinding;
+                info->has_decoration = has_decoration;
+                info->set_layout = set_layout;
+                info->set_align = set_align;
+                info->set_font = set_font;
+                info->user_writable = user_writable;
+            }
         }
     }
     else
     {
         if (has_decoration || has_keybinding)
         {
+            TRACE ("Adding %s", theme_name);
             info = g_new0 (ThemeInfo, 1);
             info->path = g_strdup (theme_dir);
             info->name = g_strdup (theme_name);
@@ -953,21 +962,57 @@ keybinding_selection_changed (GtkTreeSelection * selection, gpointer data)
         {
             ThemeInfo *ti;
 
-            g_free (current_key_theme);
-            current_key_theme = new_key_theme;
-            mcs_manager_set_string (mcs_plugin->manager, "Xfwm/KeyThemeName", CHANNEL2, current_key_theme);
-            mcs_manager_notify (mcs_plugin->manager, CHANNEL2);
-            write_options (mcs_plugin);
-
             ti = find_theme_info_by_name (new_key_theme, keybinding_theme_list);
-            
+
             if (ti)
-            {
-	        loadtheme_in_treeview (ti, itf);
-                gtk_widget_set_sensitive (itf->treeview3, ti->user_writable);
-                gtk_widget_set_sensitive (itf->treeview4, ti->user_writable);
-		gtk_widget_set_sensitive (itf->del_button, ti->user_writable);
-            }
+	    {
+	        gchar *theme_file;
+
+	        theme_file = g_build_filename (ti->path, KEY_SUFFIX, KEYTHEMERC, NULL);
+	        if (g_file_test (theme_file, G_FILE_TEST_EXISTS))
+		{
+		    g_free (current_key_theme);
+		    current_key_theme = new_key_theme;
+		    mcs_manager_set_string (mcs_plugin->manager, "Xfwm/KeyThemeName", CHANNEL2, current_key_theme);
+		    mcs_manager_notify (mcs_plugin->manager, CHANNEL2);
+		    write_options (mcs_plugin);
+		    
+		    loadtheme_in_treeview (ti, itf);
+		    gtk_widget_set_sensitive (itf->treeview3, ti->user_writable);
+		    gtk_widget_set_sensitive (itf->treeview4, ti->user_writable);
+		    gtk_widget_set_sensitive (itf->del_button, ti->user_writable);
+		}
+		else
+		{
+		    g_warning ("The keytheme file doesn't exist !");
+
+		    /* refresh list */
+		    while (keybinding_theme_list)
+		      {
+			theme_info_free ((ThemeInfo *)keybinding_theme_list->data);
+			keybinding_theme_list = g_list_next (keybinding_theme_list);
+		      }
+		    g_list_free (keybinding_theme_list);
+		    
+		    g_free (current_key_theme);
+		    current_key_theme = g_strdup ("Default");
+		    keybinding_theme_list = NULL;
+		    keybinding_theme_list = read_themes (keybinding_theme_list, itf->treeview2, itf->scrolledwindow2,
+							 KEYBINDING_THEMES, current_key_theme);
+		    gtk_widget_set_sensitive (itf->treeview3, FALSE);
+		    gtk_widget_set_sensitive (itf->treeview4, FALSE);
+		    loadtheme_in_treeview (find_theme_info_by_name ("Default", keybinding_theme_list), itf);
+		    
+		    /* tell it to the mcs manager */
+		    mcs_manager_set_string (itf->mcs_plugin->manager, "Xfwm/KeyThemeName", CHANNEL2, current_key_theme);
+		    mcs_manager_notify (itf->mcs_plugin->manager, CHANNEL2);
+		    write_options (itf->mcs_plugin);
+		}
+
+		g_free (theme_file);
+	    }
+	    else
+	        g_warning ("Cannot find the keytheme !!");
         }
     }
 }
@@ -1861,6 +1906,7 @@ setup_dialog (Itf * itf)
     {
         g_warning ("Cannot find the keytheme !");
     }
+
     g_signal_connect (G_OBJECT (itf->xfwm4_dialog), "response", G_CALLBACK (cb_dialog_response), itf->mcs_plugin);
     g_signal_connect (G_OBJECT (itf->font_button), "clicked", G_CALLBACK (show_font_selection), itf);
     g_signal_connect (G_OBJECT (itf->click_focus_radio), "toggled", G_CALLBACK (cb_click_to_focus_changed), itf);
@@ -1877,7 +1923,7 @@ setup_dialog (Itf * itf)
     g_signal_connect (G_OBJECT (itf->box_move_check), "toggled", (GCallback) cb_box_move_changed, itf);
     g_signal_connect (G_OBJECT (itf->box_resize_check), "toggled", G_CALLBACK (cb_box_resize_changed), itf);
 
-    gtk_window_set_position (GTK_WINDOW (itf->xfwm4_dialog), GTK_WIN_POS_CENTER);
+    xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (itf->xfwm4_dialog));
     gtk_widget_show (itf->xfwm4_dialog);
 }
 
@@ -2210,19 +2256,25 @@ gboolean
 write_options (McsPlugin * mcs_plugin)
 {
     gchar *rcfile, *path;
-    gboolean result;
+    gboolean result = FALSE;
 
     path = g_build_filename ("xfce4", "mcs_settings", RCFILE1, NULL);
     rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);    
-    result = mcs_manager_save_channel_to_file (mcs_plugin->manager, CHANNEL1, rcfile);
+    if (G_LIKELY (rcfile == NULL))
+    {
+        result = mcs_manager_save_channel_to_file (mcs_plugin->manager, CHANNEL1, rcfile);
+        g_free (rcfile);
+    }
     g_free (path);
-    g_free (rcfile);
 
     path = g_build_filename ("xfce4", "mcs_settings", RCFILE2, NULL);
     rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);    
-    result = mcs_manager_save_channel_to_file (mcs_plugin->manager, CHANNEL2, rcfile);
+    if (G_LIKELY (rcfile == NULL))
+    {
+        result = mcs_manager_save_channel_to_file (mcs_plugin->manager, CHANNEL2, rcfile);
+        g_free (rcfile);
+    }
     g_free (path);
-    g_free (rcfile);
 
     return result;
 }
